/***********************************    AddMan  ******************************
* Java-Addressmanager - Frontend for managing personal contacts
* Copyright (c) 2002  - Thomas Ruschival t.ruschival@vivid-md.de
*		 For  license details read LICENSE
* CVS:
* $Author: ruschi $
* $Date: 2005/01/23 10:28:11 $
* $Revision: 1.3 $ 
*				 
******************************************************************************/

import java.util.*;
import org.jdom.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;

public class AddMan {
    public static long startTime; 
    public static long stopTime ; 
    /* 
	|| Symbols 
    */
  public static final String VERSION = "1.5.5";
    /** groups are displayed in the selectionlist */ 
    public static final int GROUPSELECTIONMODE = 1;
    /** contacts are displayed in the selectionlist */ 
    public static final int USERSELECTIONMODE  = 0;   	
	/** Alllication Internal DateFormat */
	public static java.text.DateFormat 	DFMT = new java.text.SimpleDateFormat("d.M.y");       

    /*
    || Property Fields
    */	
    /** main window of the application */ 
    public static Fenster win = new Fenster("AddMan-v"+VERSION);
    /** currently selected contact */ 
    public static Person Kontakt;
    /** Object to store application preferences */ 
    public static Preferences AppPrefs;
    /** switches the current selection mode */ 
    public static int Selectionmode = USERSELECTIONMODE;
    /** global list of possible phonetypes */ 
    public static String TelefonTypListe[] = {"-----","Home","Work","Mobile","Fax","Pager"} ;
    /** list of contacts  */ 
    public static Vector myUsers = new Vector(70);
    /** global list of groups */ 
    public static Hashtable GroupHash = new Hashtable();
	/** Maximum Groupid must be incremented when adding new Groups */
	public static int MaxGroupid = 0;
    /** Flag to see if we need to ask for saving before exit */
	public static boolean datachanged = false; 	
	
	/*
    || Preferences
    */
 
   /** 
	 * Specific separator that is used to list the selected email 
	 * addresses in the clipboard for pasting into email client 
	*/                                     
    public static String SEPARATOR;
    /** File to read XML-Data from */
    public static String XMLFILENAME;
	/** File to write Data */
	public static File Savefile;
    /** XML-Reader Handle */
	public static jdomData XMLHANDLE;
	/** DOM to read from (and later to write to) */
	public static Document  XMLDoc; 
                                 
    /**                                         
	 *  Main (somewhere  the program has to start
    */	
    public static void main (String[] args){
		/** Show Window */
		win.setVisible(true);
		/** Load preferences object */
		try{
			startTime = System.currentTimeMillis(); 
			AppPrefs = loadPreferences(".AddMan");
			stopTime = System.currentTimeMillis(); 
			System.out.println("Read Preferences : " + ( stopTime - startTime) +" ms"); 
		}
		catch(FileNotFoundException exc){
			win.showErrorMessage("./.AddMan - File not Found"); 
			AppPrefs = new Preferences();
			PrefsWindow.getInstance(AppPrefs);	
		}
		catch(IOException exc){
			win.showErrorMessage("[AddMan.main()] "+ exc.getMessage());
		}
		initData();
    }


    /** 
	 * Get preferences, connection to database and  data 
	*/
    public static void initData(){		
		XMLFILENAME	= AppPrefs.getFilename();
		SEPARATOR   = AppPrefs.getSeparator();
			File datafile = new File(XMLFILENAME);
			if(datafile.exists()){
				XMLHANDLE = new jdomData(datafile);
				startTime = System.currentTimeMillis();			
				try{
					XMLDoc = XMLHANDLE.readFile();
					stopTime = System.currentTimeMillis();                                                  
					System.out.println("Building DOM-Tree : " + ( stopTime - startTime) +" ms");						
					/** Load global groups */			
					Group_list(XMLDoc.getRootElement().getChild("Groups"));                           			 	
					// initialize Userlist
					startTime = System.currentTimeMillis();			
					myUsers  = getContacts(XMLDoc.getRootElement().getChild("Userlist"));
					stopTime = System.currentTimeMillis();                                                  
					System.out.println("get contact list : " + ( stopTime - startTime) +" ms");
				}
				catch(JDOMException jdexc){
					win.showErrorMessage("Error parsing the XMLFile "+ jdexc.getMessage());
				}
				catch(Exception exc){
					win.showErrorMessage("OOPS! Something is bogus...... "+ exc.getMessage());
				}
			}
			else{
				win.showErrorMessage("Datafile :" + XMLFILENAME + " does not exist !");	
				PrefsWindow.getInstance(AppPrefs);
			}
			Collections.sort(myUsers);	
			win.UserList.setListData(myUsers);
	} 

	/** 
	 * Get Userlist from DOM-TREE 
	 * @param userlist
	 * @return Vector	
	 */
	public static Vector getContacts(Element userlist){
		Vector tmpUsers = new Vector(70);	
		Iterator it = userlist.getChildren().iterator();
		while(it.hasNext()){
			Element DOMuser = (Element)it.next();
			int 	Pkey = Integer.parseInt(DOMuser.getAttributeValue("PKEY"));
			String Vname = DOMuser.getChildText("Name");
			String Nname = DOMuser.getChildText("Familyname");
			String Nick  = DOMuser.getChildText("Nickname");
			try{	
				java.util.Date Bdate = DFMT.parse(DOMuser.getChildText("Birthday"));
				Person newUser = new Person(Nname,Vname,Nick,Bdate,Pkey);
				newUser.getPostAddress(DOMuser.getChild("Mailaddresses"));
				newUser.getTelefon(DOMuser.getChild("Phones"));
				newUser.getEmail(DOMuser.getChild("Emailaddresses"));
				tmpUsers.add(newUser);
			}
			catch(java.text.ParseException exc){
				win.showErrorMessage("Error parsing Birthday [AddMan.getContacts(Element)] :"+ exc.getMessage());
			}
		}
		return tmpUsers;
	}


	/** 
	 * Load global GroupHash from DOM-Node (Grouplist) 
	 * @param grouplist  in HashTable  
	*/
	public static void Group_list(Element grouplist){
		try{
			java.util.List Grouptree = grouplist.getChildren();
			Iterator it = Grouptree.iterator();
			while(it.hasNext()){ 	
				Element GroupElement = (Element)it.next();
				String Name = GroupElement.getAttributeValue("Name");
				int pkey    = Integer.parseInt(GroupElement.getAttributeValue("PKEY"));
				/* 
				 * Save maximum Groupid, needed for generation of new Groups as without Database
				 * No new values can be obtained, so every new Group would have PKEY=0.
				 * if more Groups have PKEY=0, only the last one would be saved in Group_list
				 */ 	
				MaxGroupid = (MaxGroupid > pkey)? MaxGroupid : pkey;
				Group tmpGroup = new Group(pkey,Name);
				GroupHash.put(new Integer(GroupElement.getAttributeValue("PKEY")),tmpGroup);			
			}
		}
		catch(NullPointerException npexc){
			win.showErrorMessage("NullPointer [AddMan.GroupList(org.jdom.Element)]");
		}
	}
	


	/** 
	 * return global groups as ArrayList, required for display in Jlists 
	*/
	public static ArrayList getGroups(){
		ArrayList groups = new ArrayList(20);
		for(Enumeration en = GroupHash.elements(); en.hasMoreElements();){
			groups.add((Group) en.nextElement());
		}
		return groups;	
	}
	
	
	/** 
	 * Method to save Data XML-File opens its own JFilechooser 
	*/
	public static void savetoFile(){
		try{
			Savefile = win.showSaveFileChooser();
			jdomData writer = new jdomData(Savefile);
			writer.buildDom(myUsers);
			writer.writeXMLfile();
		}
		catch(IOException ioexc){
			win.showErrorMessage("[AddMan.savetoFile()]"+ ioexc.getMessage());
		}
		catch(NullPointerException npexc){
			win.showErrorMessage("[AddMan.savetoFile()]"+ npexc.getMessage());
		}
	}

	/** 
	* dirty hack Method to save Data XML-File opens its own JFilechooser 
	*/
	public static void generatePhoneList(){
 		try{
 			File plist = win.showSaveFileChooser();
			FileWriter fo = new FileWriter(plist);
 			for(Enumeration userenum=myUsers.elements() ; userenum.hasMoreElements();){
				try{
					Person myUser = (Person) userenum.nextElement();
					fo.write(myUser.getNname()+" "+myUser.getVname()+"\t\t");
					ArrayList Phones = myUser.getTelefon();
					for(int i=0 ; i < Phones.size() ; i++){	
						fo.write("|\t"+((Telefon)Phones.get(i)).getNummer());	
					}
				}
				catch(IndexOutOfBoundsException abexc){}
				fo.write("\n");
				fo.flush();
			}
		}
 		catch(IOException ioexc){
 			win.showErrorMessage("[AddMan.generatePhoneList()]"+ ioexc.getMessage());
 		}
	}


	
	/** 
	 *  write XML File 
	*/
	public static void save(boolean arg){
	    if(arg){
			if(!XMLFILENAME.equals("")){
				jdomData writer = new jdomData(new File(XMLFILENAME));
				writer.buildDom(myUsers);
				try{
					writer.writeXMLfile();
					datachanged=false;
				}
				catch(IOException ioexc){
					win.showErrorMessage("[AddMan.savetoFile()]"+ ioexc.getMessage());
				}
			}
			else{
				savetoFile();
			}	
		}
	}


    /** 
	 * Get preferences from file 
	*/
    public static Preferences loadPreferences(String filename)
		throws IOException {
		Preferences myobject = null;
		try{
			FileInputStream 	fs 	= new FileInputStream (filename);
			ObjectInputStream  	os 	= new ObjectInputStream (fs);
			myobject =(Preferences)os.readObject();
			os.close();		
		}
		catch(ClassNotFoundException exc){
			AddMan.win.showErrorMessage("[AddMan.loadPreferences()]" + exc.getMessage());
		}	
		return myobject;
    }
}
	
