/***********************************  DBconn  *********************************
* Java-Addressmanager - Frontend for managing personal contacts
* Copyright (c) 2002  - Thomas Ruschival t.ruschival@vivid-md.de
*		 For  license details read LICENSE
* CVS:
* $Author: ruschi $
* $Date: 2004/09/14 15:33:04 $
* $Revision: 1.2 $ 
*				 
*******************************************************************************/

import java.sql.*;
import java.util.Hashtable;
import java.io.Serializable;

public class DBconn implements Serializable{	
    /** Call for jdbc:oracle:thin:@host:port:source */
    public static final String THIN_DRIVER = "jdbc:oracle:thin:@";
    /** Call for jdbc:oracle:oci:@source*/
    public static final String OCI_DRIVER  = "jdbc:oracle:oci8:@";
    /** Call for MySQL Driver */
    public static final String MYSQL_DRIVER= "jdbc:mysql:";
    /** Driver that is finally used */
    private String DriverType;
    /** Connection String (Host:port:source) or TNS-name needed by oci-driver */
    private String DSN;
    /** Database User */
    private String User;
    /** Password for Database logon */
    private String Passwd; 
    /** Connection Object */
    private Connection Conn = null;
    /** HashTable for Prepared Statements */
    private Hashtable Queries = new Hashtable(); 

    /** get database Connection - with all settings manually set */
    public DBconn(String myuser, String mypasswd, 
				  String myhost, String myport, 
				  String mysource, String myDriver, boolean autocommit) 
		throws SQLException{	
		this.User 	= myuser;
		this.Passwd = mypasswd;
		this.DSN	= myhost+":"+ myport +":"+ mysource ;
		this.DriverType = myDriver;		
		openConn();
		setAutoCommit(autocommit);
    }
	
    /** 
	 * get database Connection 
    */
    public DBconn(){
    }

    /** 
	 *	get Database Connection - 3 arguments are enough to get a oci connection 
    */
    public DBconn(String myuser, String mypasswd, String mysource) 
		throws SQLException{
		this.User 	= myuser;
		this.Passwd = mypasswd;
		this.DSN	= mysource ;	
		this.DriverType = OCI_DRIVER;
		openConn();
		setAutoCommit(false);
    }

    /** 
	 * open connection 
    */
    public void openConn() throws SQLException{
	this.Conn = DriverManager.getConnection(this.DriverType+this.DSN,  
						this.User,
						this.Passwd);		
    }
	
    /** 
	switch Autocommitmodus of Connection 
    */
    public void setAutoCommit(boolean auto)
	throws SQLException{
	this.Conn.setAutoCommit(auto);
    }

    /** 
	 * return private Connection Object 
    */
    public Connection getConn(){
	return this.Conn;
    }
	
    /** 
	 * Set DriverType manually 
    */
    public void setDriverType(String newdriver){
	DriverType = newdriver;
    }
	
    /** 
	 * Set DSN manually 
    */
    public void setDSN(String newdsn){
	DSN = newdsn;
    }
	
    /** 
	 * Set User manually 
    */
    public void setUser(String newuser){
	User = newuser;
    }
	
    /** 
	 * Set Password manually 
    */
    public void setPass(String newpass){
	Passwd  = newpass;
    }
	
    /** 
	 * Register a Prepared Statement
    */
    public void registerStatement(String key , PreparedStatement stmnt){
	Queries.put(key,stmnt);
    }	

    /**
	 *   Get a already Prepared Statment
    */
    public PreparedStatement getStatement(String key){
	return (PreparedStatement)Queries.get(key);
    }	
}
