/***********************************    Person   *********************************
* Java-Addressmanager - Frontend for managing personal contacts
* Copyright (c) 2002  - Thomas Ruschival t.ruschival@vivid-md.de
*		 For  license details read LICENSE
* CVS:
* $Author: ruschi $
* $Date: 2004/09/14 15:33:04 $
* $Revision: 1.2 $ 
*				 
*******************************************************************************/

import java.util.*;
import org.jdom.Element;
import java.sql.*;

public class Person implements Comparable {
/**
 Fields of Person
*/
	/** Unique ID = Primary Key */
	private int PKEY;
	/**  Surname */
	private String Nname;
	/** given Name */
	private String Vname;
	/** Nickname */
	private String Nickname;
	/**  Birthday */
	private java.util.Date Bday = new java.util.Date();
	/** ArrayList of Postadresse */
	private ArrayList PostAddress = new ArrayList(5);
	/** ArrayList of email */
	private ArrayList Email = new ArrayList(5);
	/** ArrayList of Phone */
	private ArrayList Phone = new ArrayList(5);
	/** Boolean indicates if values changed */
	public boolean Modified = false;
	private Calendar calendar = new GregorianCalendar();



/** Initalize new Person with data from Database */
	public Person(String myNname, String myVname, String myNick,
				java.util.Date bdate, int myPkey){
		PKEY 	 = myPkey ;
		Nname 	 = myNname;
		Vname	 = myVname;
		Bday     = bdate;
		Nickname = myNick ; 
		calendar.setTime(Bday);

	}

/** If new person is initialized it has no Database PKEY, assign 0 */
	public Person(){
		Nname = "Surname";
		Vname = "Given";
		PKEY  = 0;
	}
    /*
    || Interface Comparable 	
    */
    public int compareTo(Object other){
		return this.toString().compareTo(other.toString());
    }

/*
|| Methoden zur Kapselung der Daten 
*/
	/**  get database primary key as identifier */
	public int getPKEY(){
		return this.PKEY;
	}
	
	/**  get private surname for display in lists */
	public String toString(){
		return getNname() + ", " + getVname() ;	
	}
	
	/**  get private surname field */
	public String getNname(){
		return this.Nname ;	
	}
	/**  get private given name field */
	public String getVname(){
		return this.Vname ;	
	}
	/**  get private Nickname field */
	public String getNickname(){
		return this.Nickname;	
	}
	
	/**  get private Birthday field */
	public String getBday(){
		//return Bday.toString();	
	   	return getBday_day()+"."+getBday_month()+"."+getBday_year();
	}
		
	/**  get Birthday day */
	public String getBday_day(){
		return Integer.toString(calendar.get(Calendar.DATE));
	}
	/**  get Birthday month */
	public String getBday_month(){
		int monat = calendar.get(Calendar.MONTH); 
		monat = monat+1;
		return Integer.toString(monat);
	}
	
	/**  get Birthday year */
	public String getBday_year(){
		return Integer.toString(calendar.get(Calendar.YEAR));	
	}


	/** return Postaddress if it is already set */
	public ArrayList getPostAddress(){
		return this.PostAddress;
	}

	/** return Emaillist */
	public ArrayList getEmail(){
		return this.Email;
	}

	/** return Telefonliste */
	public ArrayList getTelefon(){
		return this.Phone;
	}
	

	/**  change surname field */
	public void setNname(String myNname){
		this.Nname = myNname ;
		this.Modified = true;
	}
	/**  change given field */
	public void setVname(String myVname){
		this.Vname = myVname;
		this.Modified = true;
	}
	/**  change Nickname field */
	public void setNickname(String myNick){
		this.Nickname = myNick;	
		this.Modified = true;
	}
	/**  change Birthday field */
	public void setBday(java.util.Date newdate){
		//	this.Bday = newdate;
	   	calendar.setTime(newdate);
		this.Modified = true;
	}

	
	/** Read all Groups */
	public ArrayList getGroups(){
		ArrayList myGroups = new ArrayList(10);
		// Check if this.Email has been initialized if not, no Groups available
		if (this.Email == null) {
			return myGroups;
		}
		/** Get the Groups of all email, ensure no repeat */
		/** First: get all email */
		for(int i = 0 ; i < this.Email.size(); i++){
			/** Second: get all Groups of an Email */
			for(int j =0 ; j < (((Email)Email.get(i)).getGroups()).size(); j++){
				/** Avoid repeating groupnames */
				if(! myGroups.contains((((Email)Email.get(i)).getGroups()).get(j))){
					myGroups.add((((Email)Email.get(i)).getGroups()).get(j));
				}
			}	
		}			
		return myGroups;
	}
	
	/** 
	 * get the first email entry as default email 
	*/
	public Email getDefaultEmail() {
		Email myMail;
		try{
			myMail = (Email)Email.get(0);
		}
		catch(IndexOutOfBoundsException gexc){
		    myMail = null;
		}
		return myMail;
	}


	/*
	 Methods to modify attributes
	*/
	/** add email entity */
	public void addEmail(Email mymail){
	   	this.Email.add(mymail);
		this.Modified = true;
	}
	/** add Phone entity */
	public void addPhone(Telefon mytelefon){
		this.Phone.add(mytelefon);
		this.Modified = true;
	}
	/** delete Mail address entity */
	public void addAnschrift(Postadresse myadr){
		this.PostAddress.add(myadr);
		this.Modified = true;
	}

	/*
	  Methods to delete attribute values 
	*/
	/** delete email entity */
	public void removeEmail(int i){
		this.Email.remove(i);
		this.Modified = true;
	}
	/** delete phone entity */
	public void removePhone(int i){
		this.Phone.remove(i);
		this.Modified = true;
	}
	/** delete Mail address entity */
	public void removeAnschrift(int i){
		this.PostAddress.remove(i);
		this.Modified = true;
	}


	/*
	 * Methods to obtain data not set in Constructor from org.jdom.Element
 	*/
	
	/**
	 * Read Emails from the given DOM-Node (Emailaddresses) @param
	 */ 
	public ArrayList getEmail(Element myElement){
		try{
			List Emailaddresses = myElement.getChildren();
			Iterator it = Emailaddresses.iterator();
			Element DOMEmail;
			while (it.hasNext()){
				DOMEmail = (Element) it.next();
				String Email	= DOMEmail.getChildText("Mailto");
				String Komment	= DOMEmail.getChildText("Comment");
				int    PKEY		= Integer.parseInt(DOMEmail.getAttributeValue("PKEY"));
				Email  tmpEmail = new Email(Email,Komment,PKEY);
				tmpEmail.RegisterAtGroups(DOMEmail.getChild("Groups"));
				this.Email.add(tmpEmail);
			}
		}
		catch(NullPointerException npexc){
			Fenster.showErrorMessage("Nullpointer [Person.getEmail(org.jdom.Element)] "+ this.Vname);
		}
		return this.Email;
	}


	
	/**
	 * Read Phones from the given DOM-Node (Phones) 
	 * @param Element
	 */ 
	public ArrayList getTelefon(Element myElement){
		try{
			List Phonelist = myElement.getChildren();
			Iterator it = Phonelist.iterator();
			Element DOMPhone;
			while (it.hasNext()) {
				DOMPhone = (Element)it.next();
				String Nummer	= DOMPhone.getChildText("Number");
				String Komment	= DOMPhone.getChildText("Comment");
				int    PKEY		= Integer.parseInt(DOMPhone.getAttributeValue("PKEY"));
				int    Typ		= Integer.parseInt(DOMPhone.getChildText("Type"));
				this.Phone.add(new Telefon(Nummer,Komment,PKEY,Typ));
			}
		}
		catch(NullPointerException npexc){
			Fenster.showErrorMessage("NullPointer [Person.getTelefon(org.jdom.Element)]");
		}
	   	return this.Phone;
	}


	/**
	* Read Postaddresses from DOM-Node (Mailaddresses)  
	* @param Element
    */ 
	public ArrayList getPostAddress(Element myElement){
		try{
			List Mailaddresses = myElement.getChildren();
			Iterator it 	   = Mailaddresses.iterator();
			Element DOMMail;
			while (it.hasNext()){
				DOMMail = (Element)it.next();
				String Anschrift= DOMMail.getChildText("Street");
				String Stadt	= DOMMail.getChildText("City");
				String PLZ		= DOMMail.getChildText("Zipcode");
				String Land		= DOMMail.getChildText("Country");
				String Komment	= DOMMail.getChildText("Comment");
				int    PKEY		= Integer.parseInt(DOMMail.getAttributeValue("PKEY"));
				this.PostAddress.add(new Postadresse(Anschrift,Stadt,PKEY,  
													 Land, Komment,PLZ));
			}
		}
		catch(NullPointerException npexc){
			Fenster.showErrorMessage("NullPointer [Person.getPostAddress(org.jdom.Element)]");
		}
		return this.PostAddress;
	}
}	 















